library(Matrix)
library(NMF)

how.i.fit.the.model <- function()
{
  x <- readMM("ml-100k/u.mm")
  s <- as.matrix(x)
  apply(s, 2, function (x) all(x==0))
  apply(s, 1, function (x) all(x==0))
  s <- s[1:938,]
  my.nmf.20 <- nmf(s, 20, .options='v')
  save(my.nmf.20, file="fits/nmf20.rda")
}

in.class.demonstration <- function()
{
  load("fits/nmf20.rda")

  # give the object a convenient name for typing
  m <- my.nmf.20

  # look at the structure of the result
  str(m)

  # read the items and set up column names
  items <- read.table("ml-100k/u.item", sep="|", quote="", as.is=T)
  colnames(items) <- c("id", "title", "date", "video", "imdb", "unknown",
                       "action", "adventure", "animation", "childrens",
                       "comedy", "crime", "documentary", "drama", "fantasy",
                       "film-noir","horror", "musical", "mystery", "romance",
                       "scifi", "thriller", "war", "western")

  # show some of the components
  barplot(m@fit@W[,12])
  print.top.movies(12, m@fit, items)
  print.top.movies(13, m@fit, items)
  print.top.movies(17, m@fit, items)
  print.top.movies(20, m@fit, items)
  print.top.movies(7, m@fit, items)
  print.top.movies(6, m@fit, items)
  print.top.movies(3, m@fit, items)

  # form some predictions
  me <- numeric(1682)

  # i like terminator
  grep("Terminator", items$title)
  me <- numeric(1682)
  me[195] <- 5
  my.factors <- me %*% m@fit@W
  barplot(my.factors)
  my.prediction <- my.factors %*% t(m@fit@W)
  items$title[order(my.prediction, decreasing=T)[1:10]]

  # my favorite movie ever
  rec.movies("Graduate", m@fit, items)
  # a kids movie
  rec.movies("Snow White", m@fit, items)

  # discuss the "cold start" problem for both movies and users

  # get user information
  users <- read.table("ml-100k/u.user", sep="|", quote="", as.is=T)
  users <- users[1:938,]
  colnames(users) <- c("id", "age", "gender", "occupation", "zip code")

  # plot age by factor
  qplot(users$age, m@fit@H[12,])
  qplot(users$age, m@fit@H[13,])
  qplot(users$age, m@fit@H[6,])

  # plot gender by factor
  qplot(users$gender, m@fit@H[7,])
  qplot(users$gender, m@fit@H[20,])
}

# ----- helper functions

print.top.movies <- function(i, fit, items, n=10)
{
  cat((items$title[order(fit@W[,i], decreasing=T)[1:10]]), sep="\n")
}

rec.movies <- function(title, fit, items, n=10)
{
  me <- numeric(1682)
  me[grep(title, items$title)] <- 5
  my.factors <- me %*% fit@W
  my.prediction <- my.factors %*% t(fit@W)
  items$title[order(my.prediction, decreasing=T)[1:n]]
}
